import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:instabug_dio_interceptor/instabug_dio_interceptor.dart';
import 'package:instabug_flutter/Instabug.dart';

import 'mock_adapter.dart';

class MyInterceptor extends InstabugDioInterceptor {
  int requestCount = 0;
  int resposneCount = 0;
  int errorCount = 0;

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    requestCount++;
    super.onRequest(options, handler);
  }

  @override
  void onResponse(
      Response<dynamic> response, ResponseInterceptorHandler handler) {
    resposneCount++;
    super.onResponse(response, handler);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    errorCount++;
    super.onError(err, handler);
  }
}

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();
  WidgetsFlutterBinding.ensureInitialized();
  late Dio dio;
  late MyInterceptor instabugDioInterceptor;
  const String appToken = '068ba9a8c3615035e163dc5f829c73be';
  setUpAll(() async {
    const MethodChannel('instabug_flutter')
        .setMockMethodCallHandler((MethodCall methodCall) async {
      switch (methodCall.method) {
        case 'getTags':
          return <String>['tag1', 'tag2'];
        default:
          return null;
      }
    });
  });
  setUp(() {
    dio = Dio();
    dio.options.baseUrl = MockAdapter.mockBase;
    dio.httpClientAdapter = MockAdapter();
    final List<InvocationEvent> events = <InvocationEvent>[];
    instabugDioInterceptor = MyInterceptor();
    dio.interceptors.add(instabugDioInterceptor);
    Instabug.start(appToken, events);
  });

  test('onResponse Test', () async {
    try {
      await dio.get<dynamic>('/test');
    } on DioError {
      // ignor
    }

    expect(instabugDioInterceptor.requestCount, 1);
    expect(instabugDioInterceptor.resposneCount, 1);
    expect(instabugDioInterceptor.errorCount, 0);
  });

  test('onError Test', () async {
    try {
      await dio.get<dynamic>('/test-error');
    } on DioError {
      // ignor
    }

    expect(instabugDioInterceptor.requestCount, 1);
    expect(instabugDioInterceptor.resposneCount, 0);
    expect(instabugDioInterceptor.errorCount, 1);
  });

  test('Stress Test', () async {
    for (int i = 0; i < 1000; i++) {
      try {
        await dio.get<dynamic>('/test');
      } on DioError {
        // ignor
      }
    }
    expect(instabugDioInterceptor.requestCount, 1000);
  });
}
